@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "test.h"
 
@@ -35,7 +37,7 @@
   #define STDERR_FILENO 2
 #endif
 
-int test(char *URL)
+CURLcode test(char *URL)
 {
   CURLcode res;
   CURL *curl;
@@ -59,17 +61,10 @@ int test(char *URL)
   res = curl_easy_perform(curl);
 
   if(!res) {
-    /* we are connected, now get a HTTP document the raw way */
+    /* we are connected, now get an HTTP document the raw way */
     const char *request =
-#ifdef CURL_DOES_CONVERSIONS
-      /* ASCII representation with escape sequences for non-ASCII platforms */
-      "\x47\x45\x54\x20\x2f\x35\x35\x36\x20\x48\x54\x54\x50\x2f\x31\x2e"
-      "\x32\x0d\x0a\x48\x6f\x73\x74\x3a\x20\x6e\x69\x6e\x6a\x61\x0d\x0a"
-      "\x0d\x0a";
-#else
-      "GET /556 HTTP/1.2\r\n"
+      "GET /556 HTTP/1.1\r\n"
       "Host: ninja\r\n\r\n";
-#endif
     size_t iolen = 0;
 
     res = curl_easy_send(curl, request, strlen(request), &iolen);
@@ -82,21 +77,17 @@ int test(char *URL)
         /* busy-read like crazy */
         res = curl_easy_recv(curl, buf, sizeof(buf), &iolen);
 
-#ifdef TPF
-        sleep(1); /* avoid ctl-10 dump */
-#endif
-
         if(iolen) {
           /* send received stuff to stdout */
           if(!write(STDOUT_FILENO, buf, iolen))
             break;
         }
 
-      } while((res == CURLE_OK && iolen != 0) || (res == CURLE_AGAIN));
+      } while((res == CURLE_OK && iolen) || (res == CURLE_AGAIN));
     }
 
-    if(iolen != 0)
-      res = (CURLcode)TEST_ERR_FAILURE;
+    if(iolen)
+      res = TEST_ERR_FAILURE;
   }
 
 test_cleanup:
@@ -104,5 +95,5 @@ test_cleanup:
   curl_easy_cleanup(curl);
   curl_global_cleanup();
 
-  return (int)res;
+  return res;
 }
